@@ -128,7 +128,7 @@ extern "C" {
 #   define EV_USE_EVENTFD 0
 #  endif
 # endif
-   
+ 
 #endif
 
 #include <math.h>
@@ -156,6 +156,7 @@ extern "C" {
 # include <sys/wait.h>
 # include <unistd.h>
 #else
+# include <io.h>
 # define WIN32_LEAN_AND_MEAN
 # include <windows.h>
 # ifndef EV_SELECT_IS_WINSOCKET
@@ -166,7 +167,11 @@ extern "C" {
 /* this block tries to deduce configuration from header-defined symbols and defaults */
 
 #ifndef EV_USE_MONOTONIC
-# define EV_USE_MONOTONIC 0
+# if defined (_POSIX_MONOTONIC_CLOCK) && _POSIX_MONOTONIC_CLOCK >= 0
+#  define EV_USE_MONOTONIC 1
+# else
+#  define EV_USE_MONOTONIC 0
+# endif
 #endif
 
 #ifndef EV_USE_REALTIME
@@ -174,7 +179,11 @@ extern "C" {
 #endif
 
 #ifndef EV_USE_NANOSLEEP
-# define EV_USE_NANOSLEEP 0
+# if _POSIX_C_SOURCE >= 199309L
+#  define EV_USE_NANOSLEEP 1
+# else
+#  define EV_USE_NANOSLEEP 0
+# endif
 #endif
 
 #ifndef EV_USE_SELECT
@@ -238,11 +247,15 @@ extern "C" {
 #endif
 
 #if 0 /* debugging */
-# define EV_VERIFY 1
+# define EV_VERIFY 3
 # define EV_USE_4HEAP 1
 # define EV_HEAP_CACHE_AT 1
 #endif
 
+#ifndef EV_VERIFY
+# define EV_VERIFY !EV_MINIMAL
+#endif
+
 #ifndef EV_USE_4HEAP
 # define EV_USE_4HEAP !EV_MINIMAL
 #endif
@@ -296,12 +309,7 @@ int eventfd (unsigned int initval, int flags);
 
 /**/
 
-/* EV_VERIFY: enable internal consistency checks
- * undefined or zero: no verification done or available
- * 1 or higher: ev_loop_verify function available
- * 2 or higher: ev_loop_verify is called frequently
- */
-#if EV_VERIFY >= 1
+#if EV_VERIFY >= 3
 # define EV_FREQUENT_CHECK ev_loop_verify (EV_A)
 #else
 # define EV_FREQUENT_CHECK do { } while (0)
@@ -559,6 +567,9 @@ ev_sleep (ev_tstamp delay)
       tv.tv_sec  = (time_t)delay;
       tv.tv_usec = (long)((delay - (ev_tstamp)(tv.tv_sec)) * 1e6);
 
+      /* here we rely on sys/time.h + sys/types.h + unistd.h providing select */
+      /* somehting nto guaranteed by newer posix versions, but guaranteed */
+      /* by older ones */
       select (0, 0, 0, 0, &tv);
 #endif
     }
@@ -702,13 +713,13 @@ fd_reify (EV_P)
 #if EV_SELECT_IS_WINSOCKET
       if (events)
         {
-          unsigned long argp;
+          unsigned long arg;
           #ifdef EV_FD_TO_WIN32_HANDLE
             anfd->handle = EV_FD_TO_WIN32_HANDLE (fd);
           #else
             anfd->handle = _get_osfhandle (fd);
           #endif
-          assert (("libev only supports socket fds in this configuration", ioctlsocket (anfd->handle, FIONREAD, &argp) == 0));
+          assert (("libev only supports socket fds in this configuration", ioctlsocket (anfd->handle, FIONREAD, &arg) == 0));
         }
 #endif
 
@@ -771,7 +782,7 @@ fd_ebadf (EV_P)
 
   for (fd = 0; fd < anfdmax; ++fd)
     if (anfds [fd].events)
-      if (!fd_valid (fd) == -1 && errno == EBADF)
+      if (!fd_valid (fd) && errno == EBADF)
         fd_kill (EV_A_ fd);
 }
 
@@ -940,27 +951,13 @@ void inline_size
 reheap (ANHE *heap, int N)
 {
   int i;
+
   /* we don't use floyds algorithm, upheap is simpler and is more cache-efficient */
   /* also, this is easy to implement and correct for both 2-heaps and 4-heaps */
   for (i = 0; i < N; ++i)
     upheap (heap, i + HEAP0);
 }
 
-#if EV_VERIFY
-static void
-checkheap (ANHE *heap, int N)
-{
-  int i;
-
-  for (i = HEAP0; i < N + HEAP0; ++i)
-    {
-      assert (("active index mismatch in heap", ev_active (ANHE_w (heap [i])) == i));
-      assert (("heap condition violated", i == HEAP0 || ANHE_at (heap [HPARENT (i)]) <= ANHE_at (heap [i])));
-      assert (("heap at cache mismatch", ANHE_at (heap [i]) == ev_at (ANHE_w (heap [i]))));
-    }
-}
-#endif
-
 /*****************************************************************************/
 
 typedef struct
@@ -992,7 +989,7 @@ void inline_speed
 fd_intern (int fd)
 {
 #ifdef _WIN32
-  int arg = 1;
+  unsigned long arg = 1;
   ioctlsocket (_get_osfhandle (fd), FIONBIO, &arg);
 #else
   fcntl (fd, F_SETFD, FD_CLOEXEC);
@@ -1486,6 +1483,7 @@ loop_fork (EV_P)
 }
 
 #if EV_MULTIPLICITY
+
 struct ev_loop *
 ev_loop_new (unsigned int flags)
 {
@@ -1515,40 +1513,106 @@ ev_loop_fork (EV_P)
 }
 
 #if EV_VERIFY
-static void
-array_check (W **ws, int cnt)
+static void noinline
+verify_watcher (EV_P_ W w)
 {
-  while (cnt--)
-    assert (("active index mismatch", ev_active (ws [cnt]) == cnt + 1));
+  assert (("watcher has invalid priority", ABSPRI (w) >= 0 && ABSPRI (w) < NUMPRI));
+
+  if (w->pending)
+    assert (("pending watcher not on pending queue", pendings [ABSPRI (w)][w->pending - 1].w == w));
 }
 
-static void
-ev_loop_verify (EV_P)
+static void noinline
+verify_heap (EV_P_ ANHE *heap, int N)
 {
   int i;
 
-  checkheap (timers, timercnt);
-#if EV_PERIODIC_ENABLE
-  checkheap (periodics, periodiccnt);
-#endif
+  for (i = HEAP0; i < N + HEAP0; ++i)
+    {
+      assert (("active index mismatch in heap", ev_active (ANHE_w (heap [i])) == i));
+      assert (("heap condition violated", i == HEAP0 || ANHE_at (heap [HPARENT (i)]) <= ANHE_at (heap [i])));
+      assert (("heap at cache mismatch", ANHE_at (heap [i]) == ev_at (ANHE_w (heap [i]))));
 
-#if EV_IDLE_ENABLE
-  for (i = NUMPRI; i--; )
-    array_check ((W **)idles [i], idlecnt [i]);
-#endif
-#if EV_FORK_ENABLE
-  array_check ((W **)forks, forkcnt);
-#endif
-  array_check ((W **)prepares, preparecnt);
-  array_check ((W **)checks, checkcnt);
-#if EV_ASYNC_ENABLE
-  array_check ((W **)asyncs, asynccnt);
-#endif
+      verify_watcher (EV_A_ (W)ANHE_w (heap [i]));
+    }
+}
+
+static void noinline
+array_verify (EV_P_ W *ws, int cnt)
+{
+  while (cnt--)
+    {
+      assert (("active index mismatch", ev_active (ws [cnt]) == cnt + 1));
+      verify_watcher (EV_A_ ws [cnt]);
+    }
 }
 #endif
 
+void
+ev_loop_verify (EV_P)
+{
+#if EV_VERIFY
+  int i;
+  WL w;
+
+  assert (activecnt >= -1);
+
+  assert (fdchangemax >= fdchangecnt);
+  for (i = 0; i < fdchangecnt; ++i)
+    assert (("negative fd in fdchanges", fdchanges [i] >= 0));
+
+  assert (anfdmax >= 0);
+  for (i = 0; i < anfdmax; ++i)
+    for (w = anfds [i].head; w; w = w->next)
+      {
+        verify_watcher (EV_A_ (W)w);
+        assert (("inactive fd watcher on anfd list", ev_active (w) == 1));
+        assert (("fd mismatch between watcher and anfd", ((ev_io *)w)->fd == i));
+      }
+
+  assert (timermax >= timercnt);
+  verify_heap (EV_A_ timers, timercnt);
+
+#if EV_PERIODIC_ENABLE
+  assert (periodicmax >= periodiccnt);
+  verify_heap (EV_A_ periodics, periodiccnt);
 #endif
 
+  for (i = NUMPRI; i--; )
+    {
+      assert (pendingmax [i] >= pendingcnt [i]);
+#if EV_IDLE_ENABLE
+      assert (idleall >= 0);
+      assert (idlemax [i] >= idlecnt [i]);
+      array_verify (EV_A_ (W *)idles [i], idlecnt [i]);
+#endif
+    }
+
+#if EV_FORK_ENABLE
+  assert (forkmax >= forkcnt);
+  array_verify (EV_A_ (W *)forks, forkcnt);
+#endif
+
+#if EV_ASYNC_ENABLE
+  assert (asyncmax >= asynccnt);
+  array_verify (EV_A_ (W *)asyncs, asynccnt);
+#endif
+
+  assert (preparemax >= preparecnt);
+  array_verify (EV_A_ (W *)prepares, preparecnt);
+
+  assert (checkmax >= checkcnt);
+  array_verify (EV_A_ (W *)checks, checkcnt);
+
+# if 0
+  for (w = (ev_child *)childs [chain & (EV_PID_HASHSIZE - 1)]; w; w = (ev_child *)((WL)w)->next)
+  for (signum = signalmax; signum--; ) if (signals [signum].gotsig)
+# endif
+#endif
+}
+
+#endif /* multiplicity */
+
 #if EV_MULTIPLICITY
 struct ev_loop *
 ev_default_loop_init (unsigned int flags)
@@ -1622,8 +1686,6 @@ call_pending (EV_P)
 {
   int pri;
 
-  EV_FREQUENT_CHECK;
-
   for (pri = NUMPRI; pri--; )
     while (pendingcnt [pri])
       {
@@ -1635,10 +1697,9 @@ call_pending (EV_P)
 
             p->w->pending = 0;
             EV_CB_INVOKE (p->w, p->events);
+            EV_FREQUENT_CHECK;
           }
       }
-
-  EV_FREQUENT_CHECK;
 }
 
 #if EV_IDLE_ENABLE
@@ -1700,6 +1761,7 @@ void inline_size
 periodics_reify (EV_P)
 {
   EV_FREQUENT_CHECK;
+
   while (periodiccnt && ANHE_at (periodics [HEAP0]) < ev_rt_now)
     {
       ev_periodic *w = (ev_periodic *)ANHE_w (periodics [HEAP0]);
@@ -1715,7 +1777,6 @@ periodics_reify (EV_P)
 
           ANHE_at_cache (periodics [HEAP0]);
           downheap (periodics, periodiccnt, HEAP0);
-          EV_FREQUENT_CHECK;
         }
       else if (w->interval)
         {
@@ -1850,6 +1911,12 @@ ev_unref (EV_P)
   --activecnt;
 }
 
+void
+ev_now_update (EV_P)
+{
+  time_update (EV_A_ 1e100);
+}
+
 static int loop_done;
 
 void
@@ -1861,6 +1928,10 @@ ev_loop (EV_P_ int flags)
 
   do
     {
+#if EV_VERIFY >= 2
+      ev_loop_verify (EV_A);
+#endif
+
 #ifndef _WIN32
       if (expect_false (curpid)) /* penalise the forking check even more */
         if (expect_false (getpid () != curpid))
@@ -2510,6 +2581,12 @@ infy_fork (EV_P)
 
 #endif
 
+#ifdef _WIN32
+# define EV_LSTAT(p,b) _stati64 (p, b)
+#else
+# define EV_LSTAT(p,b) lstat (p, b)
+#endif
+
 void
 ev_stat_stat (EV_P_ ev_stat *w)
 {
@@ -2913,7 +2990,7 @@ once_cb (EV_P_ struct ev_once *once, int revents)
   void (*cb)(int revents, void *arg) = once->cb;
   void *arg = once->arg;
 
-  ev_io_stop (EV_A_ &once->io);
+  ev_io_stop    (EV_A_ &once->io);
   ev_timer_stop (EV_A_ &once->to);
   ev_free (once);
 
