@@ -90,6 +90,10 @@ typedef double ev_tstamp;
 /*****************************************************************************/
 
 #if EV_STAT_ENABLE
+# ifdef _WIN32
+#  include <time.h>
+#  include <sys/types.h>
+# endif
 # include <sys/stat.h>
 #endif
 
@@ -452,6 +456,7 @@ void ev_loop_fork (EV_P);
 void ev_loop_verify (EV_P);
 
 ev_tstamp ev_now (EV_P); /* time w.r.t. timers and the eventloop, updated after each poll */
+void ev_now_update (EV_P);
 
 #else
 
